import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:jobportal/logic/fetch.dart';
import 'package:intl/intl.dart';
import 'package:jobportal/services/global_variables.dart';

import '../../data/job_interview.dart';
import 'company_interview_details.dart';

class CompanyAllInterviews extends StatefulWidget {
  CompanyAllInterviews({super.key});

  @override
  State<CompanyAllInterviews> createState() => _CompanyAllInterviewsState();
}

class _CompanyAllInterviewsState extends State<CompanyAllInterviews> {
  List<JobInterview>? _interviews;
  List<JobInterview>? _allInterviews;

  DateTime? _selectedDate;
  bool _isSortAsc = true;

  @override
  void initState() {
    super.initState();
    _fetchJobInterviews();
  }

  Future<void> _fetchJobInterviews() async {
    _interviews = await Fetch.getAllInterviewsByCompanyId(identifier.id);
    _allInterviews = _interviews;
    setState(() {});
  }

  Future<void> _updatedJobInterviews() async {
    if (_selectedDate == null) {
      _interviews = _allInterviews;
    } else {
      _interviews = await Fetch.getAllInterviewsByCompanyIdAndDate(
          identifier.id, _selectedDate!);
    }
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: Scaffold(
        backgroundColor: Colors.grey.shade300,
        body: Scaffold(
          backgroundColor: Colors.transparent,
          appBar: AppBar(
            backgroundColor: Colors.orange,
            title: const Center(child: Text("job interviews")),
          ),
          body: _buildUI(context),
        ),
      ),
    );
  }

  Future<void> _selectDate(BuildContext context) async {
    final DateTime? picked = await showDatePicker(
      context: context,
      initialDate: _selectedDate ?? DateTime.now(),
      firstDate: DateTime(2010),
      lastDate: DateTime(2025),
    );
    if (picked != null && picked != _selectedDate) {
      setState(() {
        _selectedDate = picked;
        _updatedJobInterviews();
      });
    } else {
      setState(() {
        _selectedDate = null;
        _updatedJobInterviews();
      });
    }
  }

  Widget _buildUI(BuildContext context) {
    return Column(
      children: [
        Padding(
          padding: const EdgeInsets.all(16.0),
          child: Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              ElevatedButton.icon(
                onPressed: () {
                  _selectDate(context);
                },
                icon: Icon(_selectedDate == null
                    ? Icons.calendar_today
                    : Icons.calendar_today_outlined),
                label: Text(_selectedDate == null
                    ? 'Select Date'
                    : DateFormat('yyyy-MM-dd').format(_selectedDate!)),
              ),
              const Spacer(),
            ],
          ),
        ),
        if (_interviews != null)
          Expanded(
            child: SafeArea(
              child: SizedBox(
                child: SingleChildScrollView(
                  scrollDirection: Axis.horizontal,
                  child: SingleChildScrollView(
                    scrollDirection: Axis.horizontal,
                    child: DataTable(
                      headingRowColor: WidgetStateProperty.all(Colors.black12),
                      dataRowColor: WidgetStateProperty.all(Colors.white),
                      columnSpacing: 10.0,
                      dividerThickness: 2.0,
                      columns: _createColumns(),
                      rows: _createRows(),
                    ),
                  ),
                ),
              ),
            ),
          )
        else
          const Center(child: CircularProgressIndicator()),
      ],
    );
  }

  List<DataColumn> _createColumns() {
    return [
      DataColumn(
          label: const Text("Applicant"),
          onSort: (columnIndex, _) {
            setState(() {
              if (_isSortAsc) {
                _interviews!
                    .sort((a, b) => a.jobseekerName.compareTo(b.jobseekerName));
              } else {
                _interviews!
                    .sort((a, b) => b.jobseekerName.compareTo(a.jobseekerName));
              }
              _isSortAsc = !_isSortAsc;
            });
          }),
      DataColumn(
          label: const Text("Specification"),
          onSort: (columnIndex, _) {
            setState(() {
              if (_isSortAsc) {
                _interviews!.sort((a, b) => a.jobseekerSpecificationName
                    .compareTo(b.jobseekerSpecificationName));
              } else {
                _interviews!.sort((a, b) => b.jobseekerSpecificationName
                    .compareTo(a.jobseekerSpecificationName));
              }
              _isSortAsc = !_isSortAsc;
            });
          }),
      DataColumn(
          label: const Text("Date"),
          onSort: (columnIndex, _) {
            setState(() {
              if (_isSortAsc) {
                _interviews!.sort((a, b) => a.date.compareTo(b.date));
              } else {
                _interviews!.sort((a, b) => b.date.compareTo(a.date));
              }
              _isSortAsc = !_isSortAsc;
            });
          }),
      DataColumn(
          label: Text("Time"),
          onSort: (columnIndex, _) {
            setState(() {
              if (_isSortAsc) {
                _interviews!.sort((a, b) => a.address.compareTo(b.address));
              } else {
                _interviews!.sort((a, b) => b.address.compareTo(a.address));
              }
            });
            _isSortAsc = !_isSortAsc;
          }),
    ];
  }

  List<DataRow> _createRows() {
    return _interviews!.map((e) {
      DateTime interviewDateTime = DateTime.parse(e.date.toString());
      String formattedDate = DateFormat('yyyy-MM-dd').format(interviewDateTime);
      String formattedTime = DateFormat('HH:mm').format(interviewDateTime);
      return DataRow(
          onLongPress: () {
            _showInterviewDetails(e);
          },
          cells: [
            DataCell(Text(e.jobseekerName)),
            DataCell(Text(e.jobseekerSpecificationName)),
            DataCell(Center(child: Text(formattedDate))),
            DataCell(Center(child: Text(formattedTime))),
          ]);
    }).toList();
  }

  void _showInterviewDetails(JobInterview jobInterview) {
    showDialog(
      context: context,
      builder: (context) => AlertDialog(
        backgroundColor: Colors.black54,
        title: const Row(
          children: [
            Padding(
              padding: EdgeInsets.all(8.0),
              child: Icon(
                Icons.move_to_inbox_outlined,
                size: 36,
                color: Colors.white,
              ),
            ),
            Padding(
              padding: EdgeInsets.all(8.0),
              child: Text(
                "Move",
                style: TextStyle(color: Colors.white, fontSize: 36),
              ),
            ),
          ],
        ),
        content: const Text(
          "Do you want to show interview details?",
          style: TextStyle(color: Colors.white, fontSize: 20),
        ),
        actions: [
          TextButton(
              onPressed: () {
                Navigator.canPop(context) ? Navigator.pop(context) : Null;
              },
              child: const Text(
                "No",
                style: TextStyle(color: Colors.green, fontSize: 16),
              )),
          TextButton(
              onPressed: () {
                Navigator.canPop(context) ? Navigator.pop(context) : Null;
                Navigator.push(
                  context,
                  MaterialPageRoute(
                    builder: (context) => JobInterviewDetails(
                      jobInterview: jobInterview,
                    ),
                  ),
                );
              },
              child: const Text(
                "Yes",
                style: TextStyle(color: Colors.green, fontSize: 16),
              )),
        ],
      ),
    );
  }
}
